@@ -65,6 +65,9 @@ void tidy_up_after_search(void)
 	if (openfile->mark)
 		refresh_needed = TRUE;
 #endif
+#ifdef ENABLE_COLOR
+	recook |= perturbed;
+#endif
 }
 
 /* Prepare the prompt and ask the user what to search for.  Keep looping
@@ -657,6 +660,10 @@ ssize_t do_replace_loop(const char *needle, bool whole_word_only,
 			free(openfile->current->data);
 			openfile->current->data = altered;
 
+#ifdef ENABLE_COLOR
+			check_the_multis(openfile->current);
+			refresh_needed = FALSE;
+#endif
 			set_modified();
 			as_an_at = TRUE;
 			numreplaced++;
@@ -732,6 +739,12 @@ void ask_for_and_do_replacements(void)
 /* Go to the specified line and x position. */
 void goto_line_posx(ssize_t line, size_t pos_x)
 {
+#ifdef ENABLE_COLOR
+	if (line > openfile->edittop->lineno + editwinrows ||
+				(ISSET(SOFTWRAP) && line > openfile->current->lineno))
+		recook |= perturbed;
+#endif
+
 	for (openfile->current = openfile->filetop; line > 1 &&
 				openfile->current != openfile->filebot; line--)
 		openfile->current = openfile->current->next;
@@ -790,6 +803,12 @@ void goto_line_and_column(ssize_t line, ssize_t column, bool retain_answer,
 	if (line < 1)
 		line = 1;
 
+#ifdef ENABLE_COLOR
+	if (line > openfile->edittop->lineno + editwinrows ||
+				(ISSET(SOFTWRAP) && line > openfile->current->lineno))
+		recook |= perturbed;
+#endif
+
 	/* Iterate to the requested line. */
 	for (openfile->current = openfile->filetop; line > 1 &&
 				openfile->current != openfile->filebot; line--)
@@ -992,6 +1011,11 @@ void go_to_and_confirm(linestruct *line)
 	if (line != openfile->current) {
 		openfile->current = line;
 		openfile->current_x = 0;
+#ifdef ENABLE_COLOR
+		if (line->lineno > openfile->edittop->lineno + editwinrows ||
+					(ISSET(SOFTWRAP) && line->lineno > was_current->lineno))
+			recook |= perturbed;
+#endif
 		edit_redraw(was_current, CENTERING);
 		statusbar(_("Jumped to anchor"));
 	} else if (openfile->current->has_anchor)
